@@ -3,8 +3,8 @@
 ;; Copyright (C) 2024-2026 James Cherti | https://www.jamescherti.com/contact/
 
 ;; Author: James Cherti <https://www.jamescherti.com/contact/>
-;; Package-Version: 20260216.304
-;; Package-Revision: 29308eb50800
+;; Package-Version: 20260224.27
+;; Package-Revision: 4d9d1db23718
 ;; URL: https://github.com/jamescherti/compile-angel.el
 ;; Keywords: convenience
 ;; Package-Requires: ((emacs "26.3"))
@@ -119,7 +119,11 @@
                   ;; /lisp and /site-lisp: subdirs.el
                   "lisp/subdirs.el"
                   ;; Built-in no-byte-compile packages
-                  "/lisp/leim/leim-list.el"))
+                  "/lisp/leim/leim-list.el"
+                  ;; TODO: Find a permanent fix
+                  ;; This is important because Emacs loads the early-init.elc,
+                  ;; even if it is older than the early-init.el file
+                  "/early-init.el"))
   "A list of path suffixes used to exclude specific .el files from compilation.
 
 Example: \\='(\"suffix.el\" \"/filename.el\") This excludes any path that ends
@@ -137,36 +141,37 @@ include all extensions associated with .el files."
   :group 'compile-angel)
 
 (defcustom compile-angel-excluded-files-regexps
-  (delq nil (list "/lisp/international/.*\\.el"
-                  (when (bound-and-true-p doom-user-dir)
-                    "/doom-snippets/.*")
+  (delq nil (list
+             ;; Emacs
+             "/lisp/international/.*\\.el\\(?:\\.gz\\)?$"
 
-                  ;; Spacemacs: init.el and early-init.el
-                  (when (bound-and-true-p spacemacs-core-directory)
-                    (concat "^" (regexp-quote (expand-file-name
-                                               "init.el"
-                                               user-emacs-directory)) "$"))
-                  (when (bound-and-true-p spacemacs-core-directory)
-                    (concat "^" (regexp-quote (expand-file-name
-                                               "early-init.el"
-                                               user-emacs-directory)) "$"))
+             ;; Doom
+             (when (bound-and-true-p doom-user-dir)
+               "/doom-snippets/")
 
-                  ;; Spacemacs: Exclude Spacemacs core directory
-                  (when (bound-and-true-p spacemacs-core-directory)
-                    (concat "^" (regexp-quote (file-name-as-directory
-                                               (expand-file-name
-                                                spacemacs-core-directory)))))
-
-                  ;; Spacemacs: Files such as ~/.spacemacs
-                  (when (bound-and-true-p dotspacemacs-filepath)
-                    (concat "^" (regexp-quote (expand-file-name
-                                               dotspacemacs-filepath)) "$"))
-
-                  ;; Spacemacs: Exclude layers directory only if core exists
-                  (when (bound-and-true-p spacemacs-core-directory)
-                    (concat "^" (regexp-quote (expand-file-name
-                                               "layers/"
-                                               user-emacs-directory))))))
+             ;; Spacemacs
+             (when (bound-and-true-p spacemacs-core-directory)
+               (concat "^" (regexp-quote (expand-file-name
+                                          "init.el"
+                                          user-emacs-directory)) "$"))
+             (when (bound-and-true-p spacemacs-core-directory)
+               (concat "^" (regexp-quote (expand-file-name
+                                          "early-init.el"
+                                          user-emacs-directory)) "$"))
+             (when (bound-and-true-p spacemacs-core-directory)
+               ;; Spacemacs: Exclude Spacemacs core directory
+               (concat "^" (regexp-quote (file-name-as-directory
+                                          (expand-file-name
+                                           spacemacs-core-directory)))))
+             (when (bound-and-true-p dotspacemacs-filepath)
+               ;; Spacemacs: Files such as ~/.spacemacs
+               (concat "^" (regexp-quote (expand-file-name
+                                          dotspacemacs-filepath)) "$"))
+             (when (bound-and-true-p spacemacs-core-directory)
+               ;; Spacemacs: Exclude layers directory only if core exists
+               (concat "^" (regexp-quote (expand-file-name
+                                          "layers/"
+                                          user-emacs-directory))))))
   "A list of regular expressions to exclude certain .el files from compilation.
 
 It is advisable to use `compile-angel-excluded-files' instead of
@@ -273,7 +278,7 @@ Every time Emacs performs a file operation (like `load', `file-exists-p',
   list.
 - This list often contains complex regexes for TRAMP (remote files), image
   libraries, archives, and encryption.
-- By setting it to 'safe (which usually leaves only one or zero entries), you
+- By setting it to \='safe (which usually leaves only one or zero entries), you
   skip all those regex checks. In a loop compiling hundreds of files, saving
   ~0.5ms per file operation adds up to noticeable seconds.
 
@@ -282,8 +287,8 @@ startup because they know exactly which files are being loaded (usually local,
 unencrypted ones).
 
 For general purpose compilation: It depends.
-- If you only compile local files: Use 'safe. It gives you free performance.
-- If you use TRAMP: You must use 'nil.
+- If you only compile local files: Use \='safe. It gives you free performance.
+- If you use TRAMP: You must use \='nil.
 
 This setting is intended to improve performance by reducing the overhead
 associated with file-related operations during compilation."
@@ -366,7 +371,7 @@ don't have associated .el files and therefore don't need compilation.")
 (defvar compile-angel--reload-after-native-compile (make-hash-table :test 'equal))
 
 (defvar compile-angel--list-processed-files (make-hash-table :test 'equal))
-(defvar compile-angel--list-compiled-features (make-hash-table :test 'eq))
+(defvar compile-angel--list-processed-features (make-hash-table :test 'eq))
 
 (defvar compile-angel--legacy-currently-compiling nil)
 
@@ -508,14 +513,16 @@ declaration is absent or not trusted under safe-local-variable rules."
 (defun compile-angel--el-file-excluded-p (el-file)
   "Check if EL-FILE matches any regex in `compile-angel-excluded-files-regexps'.
 Return non-nil if the file should be ignored, nil otherwise."
-  (when (or (and compile-angel--excluded-path-suffixes-regexps
-                 (cl-some (lambda (regex)
-                            (string-match-p regex el-file))
-                          compile-angel--excluded-path-suffixes-regexps))
-            (and compile-angel-excluded-files-regexps
-                 (cl-some (lambda (regex)
-                            (string-match-p regex el-file))
-                          compile-angel-excluded-files-regexps)))
+  (when (or
+         (and compile-angel--excluded-path-suffixes-regexps
+              (cl-some (lambda (regex)
+                         (string-match-p regex el-file))
+                       compile-angel--excluded-path-suffixes-regexps))
+
+         (and compile-angel-excluded-files-regexps
+              (cl-some (lambda (regex)
+                         (string-match-p regex el-file))
+                       compile-angel-excluded-files-regexps)))
     (compile-angel--debug-message
       "SKIP (.el file excluded with a regex): %s" el-file)
     t))
@@ -523,13 +530,12 @@ Return non-nil if the file should be ignored, nil otherwise."
 (defun compile-angel--elisp-native-compiled-p (el-file)
   "Return non-nil if EL-FILE is native-compiled and up to date.
 Return nil if it is not native-compiled or if its .eln file is out of date."
-  (let ((eln-file (and (fboundp 'comp-el-to-eln-filename)
-                       (comp-el-to-eln-filename el-file))))
-    (when (and eln-file
-               (file-newer-than-file-p eln-file el-file))
+  (when-let* ((eln-file (and (fboundp 'comp-el-to-eln-filename)
+                             (comp-el-to-eln-filename el-file))))
+    (when (file-newer-than-file-p eln-file el-file)
       t)))
 
-(defun compile-angel--native-compile (el-file)
+(defun compile-angel--native-compile-maybe (el-file)
   "Native-compile EL-FILE."
   (compile-angel--with-fast-file-ops
     (cond
@@ -542,19 +548,21 @@ Return nil if it is not native-compiled or if its .eln file is out of date."
         "Native-compilation ignored (up-to-date): %s" el-file))
 
      (t
-      (let ((el-file-abbreviated (abbreviate-file-name el-file)))
-        (compile-angel--verbose-message
-          "Async native-compilation: %s" el-file-abbreviated))
       (let ((inhibit-message (not (or (not compile-angel-verbose)
-                                      (not compile-angel-debug))))
-            ;; TODO find a way to pass the truename to this function
-            (el-file-truename (file-truename el-file)))
+                                      (not compile-angel-debug)))))
         (when (fboundp 'native-compile-async)
           (when compile-angel-reload-compiled-version
             (puthash el-file t
                      compile-angel--reload-after-native-compile))
-          (native-compile-async el-file-truename nil
-                                compile-angel-native-compile-load)))))))
+
+          (native-compile-async el-file nil
+                                compile-angel-native-compile-load)
+
+          (when (and (boundp 'comp-async-compilations)
+                     (gethash el-file comp-async-compilations))
+            (let ((el-file-abbreviated (abbreviate-file-name el-file)))
+              (compile-angel--verbose-message
+                "Async native-compilation: %s" el-file-abbreviated)))))))))
 
 (defun compile-angel--byte-compile (el-file elc-file)
   "Byte-compile EL-FILE into ELC-FILE.
@@ -577,23 +585,22 @@ Return the byte compile result."
            (prog-mode-hook nil)
            (emacs-lisp-mode-hook nil)
            (byte-compile-result
-            (let ((original-message (symbol-function 'message)))
-              (condition-case err
-                  (byte-compile-file el-file)
-                (permission-denied
-                 (progn
-                   (compile-angel--debug-message
-                     "Byte-compilation ignored: Permission denied: %s"
-                     (error-message-string err))
-                   ;; Try to native compile
-                   'byte-compile-exception-error))
-
-                (t
+            (condition-case err
+                (byte-compile-file el-file)
+              (permission-denied
+               (progn
                  (compile-angel--debug-message
-                   "Byte-compilation ignored: Error: %s"
+                   "Byte-compilation ignored: Permission denied: %s"
                    (error-message-string err))
-                 ;; Do not native compile
-                 nil)))))
+                 ;; Try to native compile
+                 'byte-compile-exception-error))
+
+              (t
+               (compile-angel--debug-message
+                 "Byte-compilation ignored: Error: %s"
+                 (error-message-string err))
+               ;; Do not native compile
+               nil))))
       byte-compile-result))))
 
 (defun compile-angel--need-compilation-p (el-file el-file-truename feature)
@@ -824,7 +831,7 @@ When NOERROR is non-nil, suppress warnings if the file is absent."
 
         (when (and compile-angel-enable-native-compile
                    do-native-compile)
-          (compile-angel--native-compile el-file)))))))
+          (compile-angel--native-compile-maybe el-file)))))))
 
 (defun compile-angel--check-parens ()
   "Check for unbalanced parentheses in the current buffer.
@@ -1095,7 +1102,9 @@ resolved file path or nil if not found."
         feature-file)))))
 
 (defun compile-angel--entry-point (el-file &optional feature nosuffix noerror)
-  "This function is called by all the :before advices."
+  "This function is called by all the :before advices.
+The arguments EL-FILE, FEATURE, NOSUFFIX, NOERROR are the same arguments as the
+`load' function."
   (compile-angel--with-fast-file-ops
     (when (or compile-angel-enable-byte-compile
               compile-angel-enable-native-compile)
@@ -1134,7 +1143,7 @@ resolved file path or nil if not found."
                (or (gethash el-file-truename compile-angel--list-processed-files)
                    (when feature-symbol
                      (gethash feature-symbol
-                              compile-angel--list-compiled-features))))
+                              compile-angel--list-processed-features))))
           (compile-angel--debug-message
             "SKIP (In the skip hash list): %s | %s" el-file feature))
 
@@ -1149,7 +1158,7 @@ resolved file path or nil if not found."
           (when compile-angel-on-load-mode-compile-once
             (puthash el-file-truename t compile-angel--list-processed-files)
             (when feature-symbol
-              (puthash feature-symbol t compile-angel--list-compiled-features)))
+              (puthash feature-symbol t compile-angel--list-processed-features)))
 
           (compile-angel--debug-message
             "SKIP (Does not need compilation): %s | %s" el-file feature))
@@ -1164,7 +1173,7 @@ resolved file path or nil if not found."
           (when compile-angel-on-load-mode-compile-once
             (puthash el-file-truename t compile-angel--list-processed-files)
             (when feature-symbol
-              (puthash feature-symbol t compile-angel--list-compiled-features)))
+              (puthash feature-symbol t compile-angel--list-processed-features)))
 
           (compile-angel--debug-message
             "Compilation ignored (in the no-byte-compile list): %s" el-file)
@@ -1175,7 +1184,7 @@ resolved file path or nil if not found."
                                         el-file feature-symbol)
           (puthash el-file-truename t compile-angel--list-processed-files)
           (when feature-symbol
-            (puthash feature-symbol t compile-angel--list-compiled-features))
+            (puthash feature-symbol t compile-angel--list-processed-features))
 
           (let ((compile-angel--legacy-currently-compiling
                  (cons el-file-truename compile-angel--legacy-currently-compiling)))
@@ -1351,6 +1360,8 @@ NEW-VALUE is the value of the variable."
                           (unless (string-prefix-p "\\'" el-file-regexp)
                             "\\'" ))
                   path-suffixes-regexp))))
+
+      ;; TODO only start after the mode starts?
       (setq compile-angel--excluded-path-suffixes-regexps
             (nreverse path-suffixes-regexp)))))
 
@@ -1414,7 +1425,7 @@ be JIT compiled."
           (maphash (lambda (el-file _value)
                      (compile-angel--debug-message
                        "Checking if Emacs really JIT Native-Compiled: %s" el-file)
-                     (compile-angel--native-compile el-file))
+                     (compile-angel--native-compile-maybe el-file))
                    compile-angel--list-jit-native-compiled-files)
         (clrhash compile-angel--list-jit-native-compiled-files)))
 
@@ -1442,7 +1453,7 @@ be JIT compiled."
                              compile-angel--no-byte-compile-files-list))
                (not (compile-angel--el-file-excluded-p el-file)))
       (if (not (compile-angel--elisp-native-compiled-p el-file))
-          t
+          t ; up-to-date
         nil))))
 
 (defun compile-angel--get-list-non-native-compiled ()
@@ -1517,6 +1528,31 @@ the corresponding .elc or .eln filenames."
 
 ;;; Functions
 
+;;;###autoload
+(defun compile-angel-exclude-file (file)
+  "Add a specific FILE to `compile-angel-excluded-files-regexps'.
+FILE is the file path to exclude from compilation."
+  (let* ((file (expand-file-name file))
+         (file-truename (file-truename file)))
+    (add-to-list 'compile-angel-excluded-files-regexps
+                 (concat "^" (regexp-quote file) "$"))
+    (unless (string= file file-truename)
+      (add-to-list 'compile-angel-excluded-files-regexps
+                   (concat "^" (regexp-quote file-truename) "$")))))
+
+;;;###autoload
+(defun compile-angel-exclude-directory (directory)
+  "Add a specific DIRECTORY to `compile-angel-excluded-files-regexps'.
+DIRECTORY is the directory path to exclude from compilation."
+  (let* ((dir (file-name-as-directory (expand-file-name directory)))
+         (dir-truename (file-name-as-directory (file-truename dir))))
+    (add-to-list 'compile-angel-excluded-files-regexps
+                 (concat "^" (regexp-quote dir)))
+    (unless (string= dir dir-truename)
+      (add-to-list 'compile-angel-excluded-files-regexps
+                   (concat "^" (regexp-quote dir-truename))))))
+
+;;;###autoload
 (defun compile-angel-report ()
   "Create a buffer listing all Elisp files that are not native compiled."
   (interactive)
